from tkinter import Radiobutton
from Tkinters.ClasesUtiles.Interfaces.ComponenteConAction import ComponenteConAction
from Tkinters.Imports.VisualPythonBasicosImports import *
from tkinter import IntVar
class _DatosDelGrupo():
    radios=[]
    listener=None
    intVar=None

class RadioButton(Radiobutton,ComponenteConAction):
    def __init__(self,contenedor,*args):
        """
        (contenedor)
        (contenedor,texto)
        (contenedor,X,Y,texto)
        (contenedor,X,Y)
        :param contenedor:
        :param args:
        """
        a=tuplaRectificada(args)
        leng=len(a)
        texto = None
        X=None
        Y=None
        if leng==1 and esString(a[0]):
            texto=a[0]
        elif leng==2 and esIntAll(a[0],a[1]):
            X=a[0]
            Y=a[1]
        elif leng == 3 and esString(a[2]) and esIntAll(a[0], a[1]):
            texto = a[2]
            X = a[0]
            Y = a[1]

        self.inicializarComponente(super(), contenedor, X, Y)
        if texto!=None:
            self.setText(texto)
        self.setGroupIntVar(IntVar(),1)
        self._onSelecionListener=None
        self.addAccion(self._llamarOnSelecionListener)
        self._datosDelGrupo=None


    def _llamarOnSelecionListener(self):
        if self._onSelecionListener != None:
            self._onSelecionListener(self, self._intVar.get(), self.isSelected())
    def setGroupIntVar(self,intVar,value=-1):
        self._intVar=intVar
        if value==-1:
            value=self._valueIntVar
        else:
            self._valueIntVar = value
        super().config(variable=intVar,value=value)

        return self
    def getIntVar(self):
        return self._intVar
    def getValeIntVar(self):
        return self._valueIntVar

    def getDatosDelGrupo(self):
        return self._datosDelGrupo
    def setDatosDelGrupo(self,datos):
        self._datosDelGrupo=datos
        return self
    def grupBy(self,radio):
        if esRadioButton(radio):
            if radio.getDatosDelGrupo()==None:
                if self._datosDelGrupo==None:
                    if (radio.getOnSelectionListener() != None):
                        self.setOnSelectionListener(radio.getOnSelectionListener())
                    elif self.getOnSelectionListener() != None:
                        radio.setOnSelectionListener(self.getOnSelectionListener())
                    self._datosDelGrupo=_DatosDelGrupo()
                    self._datosDelGrupo.listener=radio.getOnSelectionListener()
                    self._datosDelGrupo.radios=[radio,self]
                    self._datosDelGrupo.intVar=radio.getIntVar()
                    radio.setDatosDelGrupo(self._datosDelGrupo)

                    self.setGroupIntVar(radio.getIntVar(),2)

                else:
                    grup=self._datosDelGrupo
                    grup.radios.append(radio)
                    radio.setDatosDelGrupo(grup)
                    radio.setGroupIntVar(grup.intVar,len(grup.radios))
                    radio.setOnSelectionListener(grup.listener)
            else:
                grup = radio.getDatosDelGrupo()
                grup.radios.append(self)
                self.setDatosDelGrupo(grup)
                self.setGroupIntVar(grup.intVar, len(grup.radios))
                self.setOnSelectionListener(grup.listener)








            #self.setGroupIntVar(radio.getIntVar(),radio.getValeIntVar()+1)

        return self
    def isSelected(self):
        return self._intVar.get()==self._valueIntVar
    def setSelected(self):
        self._intVar.set(self._valueIntVar)
        return self
    def setOnSelectionListener(self,listener):
        """
        listener del tipo (self,volor#actual,isSelected)
        :param listener:
        :return:
        """
        self._onSelecionListener=listener
        #self.setAccion(lambda:listener(self,self._intVar.get(),self.isSelected()))

        return self
    def getOnSelectionListener(self):
        return self._onSelecionListener


def new_RadioButton_Grid(contenedor,row,column,*args):
    """
    (contenedor, row, column,texto)
    (contenedor, row, column,texto,anclaje)
    (contenedor, row, column,texto,anclaje,pading)
    (contenedor, row, column,texto,anclaje,padingX,padingY)
    (contenedor, row, column,texto,pading)
    (contenedor, row, column,texto,padingX,padingY)
    (contenedor, row, column)
    (contenedor, row, column,anclaje)
    (contenedor, row, column,anclaje,pading)
    (contenedor, row, column,anclaje,padingX,padingY)
    (contenedor, row, column,pading)
    (contenedor, row, column,padingX,padingY)
    :param contenedor:
    :param row:
    :param column:
    :param args:
    :return:
    """
    a = tuplaRectificada(args)
    leng = len(a)
    texto = None
    iniSegundaTupla=0
    if leng>0:
        if esString(a[0]):
            texto=a[0]
            iniSegundaTupla=1
    al=list(a)
    if iniSegundaTupla!=0 and leng!=0:
        a1=tuple(al[:iniSegundaTupla])
        if iniSegundaTupla!=leng:
            a2 = tuple(al[iniSegundaTupla:])
        else:
            a2=()
    else:
        a1=()
        a2=a



    return RadioButton(contenedor,texto,a1).setGrid(row,column,a2)

def esRadioButton(a):
    return isinstance(a,RadioButton)



class TipoDeAnclaje:
    def __init__(self,valor):
        self.valor=valor
    def getValor(self):
        return self.valor

NORTE=TipoDeAnclaje("n")
SUR=TipoDeAnclaje("s")
ESTE=TipoDeAnclaje("e")
OESTE=TipoDeAnclaje("w")
CENTRO=TipoDeAnclaje("")#center
NOROESTE=TipoDeAnclaje("nw")
NORESTE=TipoDeAnclaje("ne")
SUROESTE=TipoDeAnclaje("sw")
SURESTE=TipoDeAnclaje("se")

DERECHA=ESTE
ISQUIERDA=OESTE

#must be n, ne, e, se, s, sw, w, nw, or center
values=(NORTE,SUR,ESTE,OESTE,CENTRO,NOROESTE,NORESTE,SUROESTE,SURESTE)

def esTipoDeAnclaje(a):
    return isinstance(a,TipoDeAnclaje)



from Tkinters.VisualPythonBasicos import *
from Tkinters.ClasesUtiles.Componentes.Ventana import Ventana
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.Componentes.FrameVentana import FrameVentana
from Tkinters.ClasesUtiles.Componentes.FrameVentana import esFrameVentana
from Tkinters.ClasesUtiles.Componentes.Labelr import Labelr
from Tkinters.ClasesUtiles.Componentes.Labelr import esLabelr
from Tkinters.ClasesUtiles.Componentes.Labelr import new_Labelr_Grid
from Tkinters.ClasesUtiles.Componentes.CuadroDeTexto import CuadroDeTexto
from Tkinters.ClasesUtiles.Componentes.CuadroDeTexto import esCuadroDeTexto
from Tkinters.ClasesUtiles.Componentes.CuadroDeTexto import new_CuadroDeTexto_Grid
from Tkinters.ClasesUtiles.Componentes.CuadroDeTexto import new_PasswordField
from Tkinters.ClasesUtiles.Componentes.CuadroDeTexto import new_PasswordField_Grid
from Tkinters.ClasesUtiles.Componentes.AreaDeTexto import AreaDeTexto
from Tkinters.ClasesUtiles.Componentes.AreaDeTexto import esAreaDeTexto
from Tkinters.ClasesUtiles.Componentes.AreaDeTexto import new_AreaDeTexto_Grid
from Tkinters.ClasesUtiles.Componentes.Boton import Boton
from Tkinters.ClasesUtiles.Componentes.Boton import esBoton
from Tkinters.ClasesUtiles.Componentes.Boton import new_Boton_Grid
from Tkinters.ClasesUtiles.Componentes.RadioButton import RadioButton
from Tkinters.ClasesUtiles.Componentes.RadioButton import esRadioButton
from Tkinters.ClasesUtiles.Componentes.RadioButton import new_RadioButton_Grid
from Tkinters.ClasesUtiles.Componentes.CheckBox import CheckBox
from Tkinters.ClasesUtiles.Componentes.CheckBox import esCheckBox
from Tkinters.ClasesUtiles.Componentes.CheckBox import new_CheckBox_Grid
from Tkinters.ClasesUtiles.Componentes.ListBox import ListBox
from Tkinters.ClasesUtiles.Componentes.ListBox import esListBox
from Tkinters.ClasesUtiles.Componentes.ListBox import new_ListBox_Grid
from Tkinters.ClasesUtiles.Componentes.Menur import Menur
from Tkinters.ClasesUtiles.Componentes.Menur import esMenur
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeBorde
from Tkinters.ClasesUtiles.Tipos import TipoDeCursor
from Tkinters.ClasesUtiles.ConjuntoGrid import ConjuntoGrid





from Tkinters.ClasesUtiles.Interfaces.ComponenteConAction import ComponenteConAction
from Tkinters.Imports.VisualPythonBasicosImports import *
from tkinter import Checkbutton
from tkinter import BooleanVar

class CheckBox(Checkbutton,ComponenteConAction):
    def __init__(self,contenedor,*args):
        """
        (contenedor)
        (contenedor,texto)
        (contenedor,X,Y,texto)
        (contenedor,X,Y)
        (contenedor,texto,bacground)
        :param contenedor:
        :param args:
        """
        a=tuplaRectificada(args)
        leng=len(a)
        texto = None
        X=None
        Y=None
        fondo=None
        if leng==1 and esString(a[0]):
            texto=a[0]
        elif leng==2:
            if esIntAll(a[0],a[1]):
                X=a[0]
                Y=a[1]
            elif esStringAll(a[0],a[1]):
                texto=a[0]
                fondo=a[1]
        elif leng==3 and esString(a[2]) and esIntAll(a[0],a[1]):
            texto=a[2]
            X = a[0]
            Y = a[1]

        self.inicializarComponente(super(), contenedor, X, Y)
        if texto!=None:
            self.setText(texto)
        self._booleanVar=BooleanVar();
        self.config(variable=self._booleanVar,onvalue=True,offvalue=False)
        if fondo!=None:
            self.setBackgraund(fondo)
    def setSelected(self,selected):
        self._booleanVar.set(selected)
        return self
    def isSelected(self):
        return self._booleanVar.get()
    def addSelectionListener(self,listener):
        """
        listener del tipo (self,isSelected)
        :param listener:
        :return:
        """
        self.getListeners().append(lambda:listener(self,self.isSelected()))
        return self


def new_CheckBox_Grid(contenedor,row,column,*args):
    """
    (contenedor, row, column,texto)
    (contenedor, row, column,texto,anclaje)
    (contenedor, row, column,texto,anclaje,pading)
    (contenedor, row, column,texto,anclaje,padingX,padingY)
    (contenedor, row, column,texto,pading)
    (contenedor, row, column,texto,padingX,padingY)
    (contenedor, row, column)
    (contenedor, row, column,anclaje)
    (contenedor, row, column,anclaje,pading)
    (contenedor, row, column,anclaje,padingX,padingY)
    (contenedor, row, column,pading)
    (contenedor, row, column,padingX,padingY)
    :param contenedor:
    :param row:
    :param column:
    :param args:
    :return:
    """
    a = tuplaRectificada(args)
    leng = len(a)
    texto = None
    iniSegundaTupla=0
    if leng>0:
        if esString(a[0]):
            texto=a[0]
            iniSegundaTupla=1
    al=list(a)
    if iniSegundaTupla!=0 and leng!=0:
        a1=tuple(al[:iniSegundaTupla])
        if iniSegundaTupla!=leng:
            a2 = tuple(al[iniSegundaTupla:])
        else:
            a2=()
    else:
        a1=()
        a2=a



    return CheckBox(contenedor,texto,a1).setGrid(row,column,a2)


def esCheckBox(a):
    return isinstance(a,CheckBox)



from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Tipos import TipoDeCursor
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.CoordenadaGrid import CoordenadaGrid
from tkinter import StringVar

class Componente():

    def inicializarComponente(self,superclase,contenedor,X=None,Y=None):
        #println("super=",superclase)
        #println("conte=",contenedor)
        self._superClase=superclase
        if contenedor!=None and (esTk(contenedor) or esFrame(contenedor)):
            if esVentana(contenedor):
                contenedor = contenedor.getFrame()

            superclase.__init__(contenedor)

        self._contenedor=contenedor
        #self._texto = None

        if X!=None and Y!=None:
            self.setPosition(X,Y)

        self.iniVariables(superclase)
        return self
    def getSuperClase(self):
        return self._superClase
    def iniVariables(self,superclase):
        self._superClase=superclase
        self._textSize = 10
        self._textColor = "black"
        self._coordenadaGrid = None
        #self.setTextVar(StringVar())
        #self._listeners = []
        #self._setAccion()
    def setGrid(self, row, column, *args):
        """
        ( row, column)
        ( row, column,anclaje)
        ( row, column,anclaje,pading)
        ( row, column,anclaje,padingX,padingY)
        ( row, column,pading)
        ( row, column,padingX,padingY)
        :param row:
        :param column:
        :param args:
        :return:
        """
        a = args
        leng = len(a)
        #println("len ", leng)
        # println("tipe=",type(a)," es t=",esTupla(a)," es to=",isinstance(a,tuple))
        if leng == 1 and esTupla(a[0]):
            a = a[0]
            leng = len(a)
        self._coordenadaGrid = CoordenadaGrid(row, column)
        if leng == 0:
            self.grid(row=row, column=column)
        elif leng == 1:
            if TipoDeAnclaje.esTipoDeAnclaje(a[0]):
                self.grid(row=row, column=column, sticky=a[0].getValor())
            elif esInt(a[0]):
                self.grid(row=row, column=column, padx=a[0], pady=a[0])
        elif leng == 2:
            if TipoDeAnclaje.esTipoDeAnclaje(a[0]) and esInt(a[1]):
                self.grid(row=row, column=column, sticky=a[0].getValor(), padx=a[1], pady=a[1])
            elif esIntAll(a[0], a[1]):
                self.grid(row=row, column=column, padx=a[0], pady=a[1])
        elif leng == 3 and TipoDeAnclaje.esTipoDeAnclaje(a[0]) and esIntAll(a[1], a[2]):
            self.grid(row=row, column=column, sticky=a[0].getValor(), padx=a[1], pady=a[2])
        return self

    #def setText(self, texto):
    #    if texto!=None:
    #        self.config(text=texto)
    #        self._textVar.set(texto)
    #    return self

    #def getText(self):
    #    return self._textVar.get()

    def setPosition(self, X, Y):
        self.place(x=X, y=Y)
        self._coordenadaGrid = None
        return self

    def getX(self):
        return self.winfo_x()

    def getY(self):
        return self.winfo_y()

    def setX(self, x):
        self.setPosition(x, self.getY())

    def setY(self, y):
        self.setPosition(self.getX(), y)

    def setTextColor(self, color):
        self.config(fg=color)
        self._textColor = color
        return self

    def getTextColor(self):
        return self._textColor

    def setTextSize(self, tamaño):
        self.config(font=(tamaño))
        self._textSize = tamaño
        return self

    def getTextSize(self):
        return self._textSize

    def setTextFamili(self, familia):
        self.config(font=(familia, self.getTextSize()))
        return self

    def setFont(self, familia, tamaño):
        self.config(font=(familia, tamaño))
        self._textSize = tamaño
        return self

    def setTipoDeAnclaje(self, tipoDeAnclaje):
        if TipoDeAnclaje.esTipoDeAnclaje(tipoDeAnclaje):
            self.config(anchor=tipoDeAnclaje.getValor())
        return self

    def setWidth(self, width):
        self.config(width=width)
        return self

    def setHeight(self, height):
        self.config(height=height)
        return self

    def setSize(self, width, height):
        self.config(width=width, height=height)
        return self

    def setBackgraund(self, nombreColor):
        #super().config(bg=nombreColor)
        self.config(bg=nombreColor)
        return self

    def setJustificacion(self, tipoDeJustificacion):
        if TipoDeJustificacion.esTipoDeJustificacion(tipoDeJustificacion):
            self.config(justify=tipoDeJustificacion.getValor())
        return self
    def setTipoDeAnclaje_Grid(self,tipoDeAnclaje):
        if TipoDeAnclaje.esTipoDeAnclaje(tipoDeAnclaje):
            self.grid(sticky=tipoDeAnclaje.getValor())
        return self
    def setPading_Grid(self,*args):
        """
        (row,column)
        (rowYcolumn) un parametro row==column
        :param args:
        :return:
        """
        a=tuplaRectificada(args)
        leng =len(a)
        if leng==2 and esIntAll(a[0],a[1]):
            self.grid(padx=a[0],pady=a[1])
        elif leng==1 and esInt(a[0]):
            self.grid(padx=a[0], pady=a[0])
        return self
    #Nuevos Metodos
    
    def setColumnSpan_Grid(self,span):
        self.grid(columnspan=span)
        return self
    def setRowSpan_Grid(self,span):
        self.grid(rowspan=span)
        return self

    # def addAccion(self,accion):
    #     self._listeners.append(accion)
    # def addAccionListener(self,listener):
    #     """
    #     listener del tipo (self)
    #     :param listener:
    #     :return:
    #     """
    #     #self._accion=accion
    #     #super().config(command=accion)
    #     self._listeners.append(lambda:listener(self) )
    #     return self
    # def _LlamarActionListeners(self):
    #     #print("se llama")
    #     #a=0
    #     for i in self._listeners:
    #         if i!= None:
    #             i()
    #         #println("a=",a)
    #         #a += 1

    # def _setAccion(self):
    #     self.config(command=self._LlamarActionListeners)
    # def getListeners(self):
    #     return self._listeners
    def setCursor(self,tipoDeCursor):
        if TipoDeCursor.esTipoDeCursor(tipoDeCursor):
            self.config(cursor=tipoDeCursor.getValor())

        return self


def esComponente(a):
    return isinstance(a,Componente)



from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.CoordenadaGrid import CoordenadaGrid
from Tkinters.ClasesUtiles.Interfaces.Componente import Componente
from tkinter import Text
from tkinter import Scrollbar
from tkinter import END

class AreaDeTexto(Text,Componente):
    # def __init__(self,contenedor): usar setGrid
    #def __init__(self,contenedor,X,Y):
    def __init__(self, *a):
        """
        (contenedor)
        (contenedor,X,Y)
        :param a:
        """
        leng=len(a)
        if leng>0 and (esTk(a[0]) or esFrame(a[0])):
             contenedor=a[0]
             if esVentana(a[0]):
                 contenedor=a[0].getFrame()
             super().__init__(contenedor)
             self._scrollBar=Scrollbar(contenedor,command=super().yview)
             super().config(yscrollcommand=self._scrollBar.set)
        if leng==3 and esIntAll(a[1], a[2]):
            self.setPosition(a[1], a[2])
        self._textSize = 10
        self._textColor = "black"
        #self._texto =""
        self._verScrollBar=False
        self._coordenadaGrid=None

        self._superClase=super()
        self.iniVariables(super())
    #def setGrid(self,row,column):
    #def setGrid(self, row, column,anclaje=TipoDeAnclaje.CENTRO):
    def setGrid(self,row,column, *args):
        """
        ( row, column)
        ( row, column,anclaje)
        ( row, column,anclaje,pading)
        ( row, column,anclaje,padingX,padingY)
        ( row, column,pading)
        ( row, column,padingX,padingY)
        :param row:
        :param column:
        :param args:
        :return:
        """
        a=args
        leng=len(a)
      #  println("len ", leng)
        #println("tipe=",type(a)," es t=",esTupla(a)," es to=",isinstance(a,tuple))
        if leng==1 and esTupla(a[0]):
           a=a[0]
           leng = len(a)

        self._coordenadaGrid=CoordenadaGrid(row,column)
        if leng==0:
            super().grid(row=row, column=column)
        elif leng==1:
            if TipoDeAnclaje.esTipoDeAnclaje(a[0]):
                super().grid(row=row, column=column,sticky=a[0].getValor())
            elif esInt(a[0]):
                super().grid(row=row, column=column, padx=a[0],pady=a[0])
        elif leng==2:
            if TipoDeAnclaje.esTipoDeAnclaje(a[0]) and esInt(a[1]):
                super().grid(row=row, column=column,sticky=a[0].getValor(),padx=a[1],pady=a[1])
            elif esIntAll(a[0],a[1]):
                super().grid(row=row, column=column, padx=a[0],pady=a[1])
        elif leng==3 and TipoDeAnclaje.esTipoDeAnclaje(a[0]) and esIntAll(a[1],a[2]):
            super().grid(row=row, column=column, sticky=a[0].getValor(), padx=a[1], pady=a[2])

        self.actualizarScrollBar()
        return self
    def actualizarScrollBar(self):
        coor = self._coordenadaGrid
        if self._verScrollBar and coor != None:
            self._scrollBar.grid(row=coor.row, column=coor.column + 1,sticky="nsew")
    def setVerScrollBar(self,verlo):
        self._verScrollBar=verlo
        self.actualizarScrollBar()
        return self

    def getScrollBar(self):
        return self._scrollBar
    def append(self,texto):
        super().insert(END, texto)
        return self
    def setText(self, texto):
        self.clear()
        super().insert(1.0,texto)
        return self

    def getText(self):
        return super().get("1.0",END)

    def setPosition(self, X, Y):
        super().place(x=X, y=Y)
        self._coordenadaGrid=None
        return self

    def getX(self):
        return super().winfo_x()

    def getY(self):
        return super().winfo_y()

    def setX(self, x):
        self.setPosition(x, self.getY())

    def setY(self, y):
        self.setPosition(self.getX(), y)

    def setTextColor(self, color):
        self.config(fg=color)
        self._textColor = color
        return self

    def getTextColor(self):
        return self._textColor

    def setTextSize(self, tamaño):
        self.config(font=(tamaño))
        self._textSize = tamaño
        return self

    def getTextSize(self):
        return self._textSize

    def setTextFamili(self, familia):
        self.config(font=(familia, self.getTextSize()))
        return self

    def setFont(self, familia, tamaño):
        self.config(font=(familia, tamaño))
        self._textSize = tamaño
        return self

    def setTipoDeAnclaje(self, tipoDeAnclaje):
        if TipoDeAnclaje.esTipoDeAnclaje(tipoDeAnclaje):
            self.config(anchor=tipoDeAnclaje.getValor())
        return self

    def setWidth(self, width):
        self.config(width=width)
        return self

    def setHeight(self, height):
        self.config(height=height)
        return self

    def setSize(self, width, height):
        self.config(width=width, height=height)
        return self

    def setBackgraund(self, nombreColor):
        super().config(bg=nombreColor)
        return self

    def setJustificacion(self, tipoDeJustificacion):
        if TipoDeJustificacion.esTipoDeJustificacion(tipoDeJustificacion):
            super().config(justify=tipoDeJustificacion.getValor())
        return self
    def setTipoDeAnclaje_Grid(self,tipoDeAnclaje):
        if TipoDeAnclaje.esTipoDeAnclaje(tipoDeAnclaje):
            super().grid(sticky=tipoDeAnclaje.getValor())
        return self
    def setPading_Grid(self,*args):
        """
        (row,column)
        (rowYcolumn) un parametro row==column
        :param args:
        :return:
        """
        a=tuplaRectificada(args)
        leng =len(a)
        if leng==2 and esIntAll(a[0],a[1]):
            super().grid(padx=a[0],pady=a[1])
        elif leng==1 and esInt(a[0]):
            super().grid(padx=a[0], pady=a[0])
        return self
    def setTipoPassword(self,tipoPassword):
        if tipoPassword:
            super().config(show="*")
        else:
            super().config(show="")
        return self
    def clear(self):
        super().delete(1.0,END)
#------sobrescribiendo----------
    # def setTextVar(self,textVar):
    #     if esStringVar(textVar):
    #         #self.config(textvariable=textVar)
    #         #self.getSuperClase().config(textvariable=textVar)
    #         self._textVar=textVar
    #         self._texto=textVar.get()
    #     return self
    # def _setAccion(self):
    #     pass
    # def getText(self):
    #     text=super().get("1.0",END)
    #     self._textVar.set(text)
    #     return text
    # def setText(self, texto):
    #     if texto!=None:
    #         super().insert(1.0,texto)
    #         #self.config(text=texto)
    #         self._textVar.set(texto)
    #     return self
#-----------------------------------        

def new_AreaDeTexto_Grid(contenedor,row,column,*a):
    """
    (contenedor, row, column)
    (contenedor, row, column,anclaje)
    (contenedor, row, column,anclaje,pading)
    (contenedor, row, column,anclaje,padingX,padingY)
    (contenedor, row, column,pading)
    (contenedor, row, column,padingX,padingY)
    :param contenedor:
    :param row:
    :param column:
    :param a:
    :return:
    """
    #println("len a=",len(a))
    return AreaDeTexto(contenedor).setGrid(row,column,a)

def esAreaDeTexto(a):
    return isinstance(a, AreaDeTexto)



from tkinter import Frame
from Tkinters.ClasesUtiles.Tipos import TipoDeBorde
from Tkinters.ClasesUtiles.Tipos import TipoDeCursor
from Tkinters.Imports.VisualPythonBasicosImports import *

from Utiles.Utiles import *


#from Include.Tkinters.ClasesUtiles.Tipos import TipoDeBorde
#from Include.Tkinters.ClasesUtiles.Tipos import TipoDeCursor

#from Include.Tkinters.Imports.VisualPythonImports import *


#from Tipos import TipoDeBordeas
#from Tipos import TipoDeCursor

class FrameVentana(Frame):
	# def __init__(self, contenedor, ancho, alto):

	def __init__(self,*args):
		"""
		(contenedor, ancho, alto)
		()
		:param args:
		"""
		leng = len(args)
		#println("len=",leng)
		#println("a0 ",args[0]," a1",args[1]," a2",args[2])
		#println("es tk=",esTk(args[0])," es int ",esIntAll(args[1],args[2]))
		#println("leng == 3 & esTk(args[0]) and esIntAll(args[1],args[2])=",leng == 3 & esTk(args[0]) & esIntAll(args[1],args[2]))
		if leng == 3 and esTk(args[0]) and esIntAll(args[1],args[2]):
			#print("uno")
			super().__init__(args[0],width=args[1],height=args[2])
		else:
			#print("dos")
			super().__init__()
	def setSize(self,Width,Height):
		super().config(width=Width,height=Height)
		return self
	def setBackgraund(self,nombreColor):
		super().config(bg=nombreColor)
		return self
	def setTipoDeBorde(self,borde):
		if TipoDeBorde.esTipoDeBorde(borde):
			super().config(relief=borde.getValor())
		return self
	def setBordeWidth(self,ancho):
		super().config(bd=ancho)
		return self
	def setBorde(self,ancho,tipoDeBorde):
		self.setTipoDeBorde(tipoDeBorde)
		self.setBordeWidth(ancho)
		return self
	def setCursor(self,tipoDeCursor):
		if TipoDeCursor.esTipoDeCursor(tipoDeCursor):
			super().config(cursor=tipoDeCursor.getValor())

		return self
def esFrameVentana(a):
	return isinstance(a,FrameVentana)

def new_FrameVentana():
	return FrameVentana()


"""
pip3 uninstall ReneTkinter
pip3 install ReneTkinter-1.0.tar.gz
 python setup.py sdist
 
"""


#from Tkinters.VisualPython import esFrame
#from Tkinters.VisualPython import esTk

from Tkinters.VisualPythonBasicos import esFrame
from Tkinters.VisualPythonBasicos import esTk
from Tkinters.VisualPythonBasicos import esStringVar
from Tkinters.VisualPythonBasicos import append
from Tkinters.VisualPythonBasicos import esMenu
from Tkinters.VisualPythonBasicos import responderException
from Tkinters.VisualPythonBasicos import showDialogoSalir
from Tkinters.VisualPythonBasicos import showError
from Tkinters.VisualPythonBasicos import showInfo
from Tkinters.VisualPythonBasicos import showAdvertencia
from Tkinters.VisualPythonBasicos import showAceptarCancelarInf
from Tkinters.VisualPythonBasicos import showAceptarCancelarAdvertencia
from Tkinters.VisualPythonBasicos import showFileChoser
from Tkinters.VisualPythonBasicos import showDirectoryChoser
from Utiles.Utiles import *


#from Tkinters.VisualPython import esVentana
#from Tkinters.VisualPython import esFrameVentana




from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Tipos import TipoDeCursor
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.CoordenadaGrid import CoordenadaGrid
from tkinter import StringVar
from Tkinters.ClasesUtiles.Interfaces.Componente import Componente
class ComponenteConText(Componente):
	def iniVariables(self,superclase):
		super().iniVariables(superclase)
		self.setTextVar(StringVar())
	def setText(self, texto):
		#println("text=",texto)
		if texto!=None:
			self._textVar.set(texto)
			#self.config(text=texto)
		return self
	def getText(self):
		return self._textVar.get()
	def setTextVar(self,textVar):
		if esStringVar(textVar):
			self.getSuperClase().config(textvariable=textVar)
			self._textVar=textVar
			self._texto=textVar.get()
		return self
	def getTextVar(self):
		return self._textVar
	def ligar(self,elemento):
		if Componente.esComponente(elemento):
			self.setTextVar(elemento.getTextVar())
		elif esStringVar(elemento):
			self.setTextVar(elemento)
		return self



from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Tipos import TipoDeCursor
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.CoordenadaGrid import CoordenadaGrid
from tkinter import StringVar
from Tkinters.ClasesUtiles.Interfaces.ComponenteConText import ComponenteConText
class ComponenteConAction(ComponenteConText):
	def iniVariables(self,superclase):
		super().iniVariables(superclase)
		self._listeners = []
		self._setAccion()
	def addAccion(self,accion):
		self._listeners.append(accion)
	def addAccionListener(self,listener):
		"""
		listener del tipo (self)
		:param listener:
		:return:
		"""
		self._listeners.append(lambda:listener(self) )
		return self
	def _LlamarActionListeners(self):
		for i in self._listeners:
			if i!= None:
				i()
	def _setAccion(self):
		self.config(command=self._LlamarActionListeners)
	def getListeners(self):
		return self._listeners


from hmac import new

from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Componentes.CuadroDeTexto import CuadroDeTexto
from Tkinters.ClasesUtiles.Componentes.CuadroDeTexto import new_PasswordField
from Tkinters.ClasesUtiles.Componentes.Labelr import Labelr
from Tkinters.ClasesUtiles.Componentes.Boton import Boton
from Tkinters.ClasesUtiles.Componentes.AreaDeTexto import AreaDeTexto
from Tkinters.ClasesUtiles.Componentes.RadioButton import RadioButton
from Tkinters.ClasesUtiles.Componentes.CheckBox import CheckBox
from Tkinters.ClasesUtiles.Componentes.ListBox import ListBox
from Tkinters.ClasesUtiles.Interfaces.Componente import Componente
from Tkinters.ClasesUtiles.Interfaces.Componente import esComponente

class _ConfiguracionDeColumna():
    padX=None
    padY=None
    anclaje=None
class _ConfiguracionDeFila(_ConfiguracionDeColumna):
    pass


class ConjuntoGrid():
    def __init__(self,contenedor,columns):
        self.contendor=contenedor
        self.clumns=columns
        self.rows=[]
        self.configuracionesDeColumnas=[]
        self.configuracionesDeFilas=[]
        for i in range(columns):
            self.configuracionesDeColumnas.append(_ConfiguracionDeColumna())
    def actualizar(self):
        for i in range(self.clumns):
            column=self.getColumn(i)
            leng=len(column)
            #println("i=",i," colu=",column)
            for j in range(leng):
                elem=column[j]#self.rows[i][j]
                if elem!=None :
                    conf=self.configuracionesDeColumnas[i]
                    confFila=self.configuracionesDeFilas[j]
                    if conf.padX!=None and confFila.padX==None:
                        elem.grid(padx=conf.padX)
                    if conf.padY != None and confFila.padY==None:
                        elem.grid(pady=conf.padY)
                    if conf.anclaje != None and confFila.anclaje==None:
                        elem.grid(sticky=conf.anclaje.getValor())
        leng=len(self.rows)#range(leng)
        for i in range(leng):
            conf = self.configuracionesDeFilas[i]
            if conf.padX != None or conf.padY != None or conf.anclaje != None:
                for j in self.rows[i]:
                    elem=j
                    if elem != None:
                        if conf.padX != None :
                            elem.grid(padx=conf.padX)
                        if conf.padY != None :
                            elem.grid(pady=conf.padY)
                        if conf.anclaje != None :
                            elem.grid(sticky=conf.anclaje.getValor())


        return self
    def getColumn(self,indice):
        column=[]
        for i in self.rows:
            if indice<len(i):
                column.append(i[indice])
            else:
                column.append(None)
        return column
    def get(self,X,Y):
        return self.rows[X][Y]
    def setPadingColumn(self,columnIndice,padX,padY=None):
        if padY==None:
            padY=padX
        self.configuracionesDeColumnas[columnIndice].padX=padX
        self.configuracionesDeColumnas[columnIndice].padY = padY
        return self
    def setPadingColumn_Actualizar(self,columnIndice,padX,padY=None):
        self.setPadingColumn(columnIndice,padX,padY)
        self.actualizar()
        return self
    def setPadingXColumn(self,columnIndice,padX):
        self.configuracionesDeColumnas[columnIndice].padX=padX
        return self
    def setPadingXColumn_Actualizar(self,columnIndice,padX):
        self.setPadingXColumn(columnIndice,padX)
        self.actualizar()
        return self
    def setPading(self,padX,padY=None):
        if padY==None:
            padY=padX
        for i in self.configuracionesDeColumnas:
            i.padX=padX
            i.padY=padY
        return self
    def setPadingYColumn(self, columnIndice, padY):
        self.configuracionesDeColumnas[columnIndice].padY = padY
        return self

    def setPadingYColumn_Actualizar(self, columnIndice, padY):
        self.setPadingYColumn(columnIndice, padY)
        self.actualizar()
        return self
    def setAnclaje(self,columnIndice,anclaje):
        self.configuracionesDeColumnas[columnIndice].anclaje=anclaje
        return self
    def setAnclaje_Actualizar(self,columnIndice,anclaje):
        self.setAnclaje(columnIndice,anclaje)
        self.actualizar()
        return self
    #def actualizarColumna(self,indiceColumna):

    
    def addRow(self,*hijos):
        #println("hijos=",hijos)
        row=list(tuplaRectificada(hijos))
        #println("row=" , row)
        self.rows.append(row)
        leng=len(row)
        #println("leng=" , leng)
        fila=len(self.rows)-1
        for i in range(leng):
            #println("i=" , i)
            if esComponente(row[i]):
                row[i].setGrid(fila,i)
            else:
                #println("row[i]=" , row[i])
                row[i].grid(row=fila, column=i)
        self.configuracionesDeFilas.append(_ConfiguracionDeFila())
        return self
    def addRow_Actualizar(self,*hijos):
        self.addRow(tuplaRectificada(hijos))
        self.actualizar()
        return self
    def setPadingFila(self,fila,padX,padY=None):
        if padY==None:
            padY=padX
        self.configuracionesDeFilas[fila].padX=padX
        self.configuracionesDeFilas[fila].padY = padX
        return self
    def setPadingFilaX(self,fila,padX):
        self.configuracionesDeFilas[fila].padX=padX
        return self
    def setPadingFilaY(self,fila,padY):
        self.configuracionesDeFilas[fila].padY=padY
        return self
    def setPadingFila_Actual(self,padX,padY=None):
        return self.setPadingFila(len(self.rows)-1,padX,padY)
    def setPadingFilaX_Actual(self,fila,padX):
        return self.setPadingFilaX(len(self.rows)-1,padX)
    def setPadingFilaY_Actual(self,fila,padY):
        return self.setPadingFilaY(len(self.rows)-1,padY)

    # new ++++++++++++++++++++++++++++++
    def new_CuadroDeTexto(self,textColor=None):
        ent=CuadroDeTexto(self.contendor)
        if textColor!=None:
            ent.setTextColor(textColor)
        return ent
    def new_PasswordField(self):
        return new_PasswordField(self.contendor)
    def new_Labelr(self,texto,background=None,textColor=None):
        lab=Labelr(self.contendor,texto)
        if background!=None:
            lab.setBackgraund(background)
            if textColor!=None:
                lab.setTextColor(textColor)
        return lab
    def new_AreaDeTexto(self):
        return AreaDeTexto(self.contendor)
    def new_Boton(self,texto,accion):
        return Boton(self.contendor,texto,accion)
    def new_RadioButton(self,texto=None):
        return RadioButton(self.contendor,texto)
    def new_CheckBox(self,*a):
        return CheckBox(self.contendor,*a)
    def new_ListBox(self):
        return ListBox(self.contenedor)






from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.CoordenadaGrid import CoordenadaGrid
#from Tkinters.ClasesUtiles.Interfaces.Componente import Componente
from Tkinters.ClasesUtiles.Interfaces.ComponenteConAction import ComponenteConAction
from tkinter import Button

class Boton(Button,ComponenteConAction):
    #def __init__(self,contenedor,texto,accion,X=None,Y=None):
    def __init__(self,contenedor,*args):
        """
        (contenedor)
        (contenedor,texto,accion)
        (contenedor,X,Y,texto,accion)

        :param args:
        """
        a = tuplaRectificada(args)
        leng = len(a)
        texto = None
        X = None
        Y = None
        accion=None
        if leng == 2 and esString(a[0]) and esFuncion(a[1]):
            texto = a[0]
            accion= a[1]
        elif leng == 4 and esString(a[2]) and esIntAll(a[0], a[1]) and esFuncion(a[3]):
            texto = a[2]
            X = a[0]
            Y = a[1]
            accion=a[3]

        self.inicializarComponente(super(), contenedor, X, Y)
        self.setText(texto)
        self.addAccion(accion)



def esBoton(a):
    return isinstance(a,Boton)
def new_Boton_Grid(contenedor,row,column,texto,*a):
    """
    (contenedor, row, column)
    (contenedor, row, column,anclaje)
    (contenedor, row, column,anclaje,pading)
    (contenedor, row, column,anclaje,padingX,padingY)
    (contenedor, row, column,pading)
    (contenedor, row, column,padingX,padingY)
    :param contenedor:
    :param row:
    :param column:
    :param texto:
    :param a:
    :return:
    """
    return Boton(contenedor, texto).setGrid(row, column, a)


from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.CoordenadaGrid import CoordenadaGrid
from Tkinters.ClasesUtiles.Interfaces.Componente import Componente
from tkinter import Listbox
from tkinter import END

class ListBox(Listbox,Componente):
    def __init__(self,contenedor,*args):
        """
        (contenedor) (contenedor,X,Y)
        """
        a = tuplaRectificada(args)
        leng = len(a)
        texto = None
        X = None
        Y = None
        if leng == 2 and esIntAll(a[0], a[1]):
            X = a[0]
            Y= a[1]
        self.inicializarComponente(super(), contenedor, X, Y)
    def add(self,a):
        super().insert(END, a)
        return self


def esListBox(a):
    return isinstance(a,ListBox)
def new_ListBox_Grid(contenedor,row,column,*a):
    """
    (contenedor, row, column)
    (contenedor, row, column,anclaje)
    (contenedor, row, column,anclaje,pading)
    (contenedor, row, column,anclaje,padingX,padingY)
    (contenedor, row, column,pading)
    (contenedor, row, column,padingX,padingY)
    :param contenedor:
    :param row:
    :param column:
    :param texto:
    :param a:
    :return:
    """
    return ListBox(contenedor).setGrid(row, column, *a)







class TipoDeBorde:
    def __init__(self,valor):
        self.valor=valor
    def getValor(self):
        return self.valor

GROOVE=TipoDeBorde("groove")
SUNKEN=TipoDeBorde("sunken")

values=(GROOVE,SUNKEN)
def esTipoDeBorde(a):
    return isinstance(a,TipoDeBorde)

"""
from Tkinters.ClasesUtiles.Ventana import Ventana
ven=Ventana()
ven.setTitle("Nueva")
ven.setSize(600,400)
ven.setBackgraundColor("red")
ven.setBorde(25,SUNKEN)
ven.show()
"""




class TipoDeCursor:
    def __init__(self,valor):
        self.valor=valor
    def getValor(self):
        return self.valor
MANO=TipoDeCursor("hand2")
PIRATE=TipoDeCursor("pirate")
CIRCULO=TipoDeCursor("circle")
CRUZ=TipoDeCursor("cross")
values=(MANO,PIRATE)

def esTipoDeCursor(a):
    return isinstance(a,TipoDeCursor)


from tkinter import Label
from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Tipos import TipoDeAnclaje
from Tkinters.ClasesUtiles.Tipos import TipoDeJustificacion
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from Tkinters.ClasesUtiles.CoordenadaGrid import CoordenadaGrid
from Tkinters.ClasesUtiles.Interfaces.ComponenteConText import ComponenteConText

class Labelr(Label,ComponenteConText):
    # def __init__(self,contenedor,texto): usar setGrid
    # def __init__(self,contenedor,X,Y,texto):
    def __init__(self, *args):
        """
        (contenedor,texto)
        (contenedor,X,Y,texto)
        :param args:
        """
        leng = len(args)
        contenedor=None

        #if leng > 0 and (esTk(args[0]) or esFrame(args[0])):
        #    contenedor = args[0]
         #   if esVentana(args[0]):
         #       contenedor = args[0].getFrame()
        X = None
        Y= None
        texto=None
        if(leng==2  and esString(args[1])):#and esFrame(args[0])
            #super().__init__(contenedor, text=args[1])
            #self._texto = args[1]
            texto=args[1]

        elif (leng == 4  and esString(args[3]) and esIntAll(args[1], args[2])):#and (esFrame(args[0]) or esTk(args[0]))
            #super().__init__(contenedor, text=args[3])
            #self._texto = args[3]
            #self.setPosition(args[1], args[2])
            texto=args[3]
            X=args[1]
            Y=args[2]
        self.inicializarComponente(super(),args[0],X,Y)
        self.iniVariables(super())
        self.setText(texto)
        #self._textSize = 10
        #self._textColor = "black"
        #self._coordenadaGrid = None

    def setGrid(self, row, column, *args):
        """
        ( row, column)
        ( row, column,anclaje)
        ( row, column,anclaje,pading)
        ( row, column,anclaje,padingX,padingY)
        ( row, column,pading)
        ( row, column,padingX,padingY)
        :param row:
        :param column:
        :param args:
        :return:
        """
        a = args
        leng = len(a)
        #println("len ", leng)
        # println("tipe=",type(a)," es t=",esTupla(a)," es to=",isinstance(a,tuple))
        if leng == 1 and esTupla(a[0]):
            a = a[0]
            leng = len(a)
        self._coordenadaGrid = CoordenadaGrid(row, column)
        if leng == 0:
            super().grid(row=row, column=column)
        elif leng == 1:
            if TipoDeAnclaje.esTipoDeAnclaje(a[0]):
                super().grid(row=row, column=column, sticky=a[0].getValor())
            elif esInt(a[0]):
                super().grid(row=row, column=column, padx=a[0], pady=a[0])
        elif leng == 2:
            if TipoDeAnclaje.esTipoDeAnclaje(a[0]) and esInt(a[1]):
                super().grid(row=row, column=column, sticky=a[0].getValor(), padx=a[1], pady=a[1])
            elif esIntAll(a[0], a[1]):
                super().grid(row=row, column=column, padx=a[0], pady=a[1])
        elif leng == 3 and TipoDeAnclaje.esTipoDeAnclaje(a[0]) and esIntAll(a[1], a[2]):
            super().grid(row=row, column=column, sticky=a[0].getValor(), padx=a[1], pady=a[2])
        return self

    #def setText(self, texto):
     #   super().config(text=texto)
     #   self._texto = texto
     #   return self

    #def getText(self):
    #    return self._texto

    def setPosition(self, X, Y):
        super().place(x=X, y=Y)
        self._coordenadaGrid = None
        return self

    def getX(self):
        return super().winfo_x()

    def getY(self):
        return super().winfo_y()

    def setX(self, x):
        self.setPosition(x, self.getY())

    def setY(self, y):
        self.setPosition(self.getX(), y)

    def setTextColor(self, color):
        self.config(fg=color)
        self._textColor = color
        return self

    def getTextColor(self):
        return self._textColor

    def setTextSize(self, tamaño):
        self.config(font=(tamaño))
        self._textSize = tamaño
        return self

    def getTextSize(self):
        return self._textSize

    def setTextFamili(self, familia):
        self.config(font=(familia, self.getTextSize()))
        return self

    def setFont(self, familia, tamaño):
        self.config(font=(familia, tamaño))
        self._textSize = tamaño
        return self

    def setTipoDeAnclaje(self, tipoDeAnclaje):
        if TipoDeAnclaje.esTipoDeAnclaje(tipoDeAnclaje):
            self.config(anchor=tipoDeAnclaje.getValor())
        return self

    def setWidth(self, width):
        self.config(width=width)
        return self

    def setHeight(self, height):
        self.config(height=height)
        return self

    def setSize(self, width, height):
        self.config(width=width, height=height)
        return self

    def setBackgraund(self, nombreColor):
        super().config(bg=nombreColor)
        return self

    def setJustificacion(self, tipoDeJustificacion):
        if TipoDeJustificacion.esTipoDeJustificacion(tipoDeJustificacion):
            super().config(justify=tipoDeJustificacion.getValor())
        return self
    def setTipoDeAnclaje_Grid(self,tipoDeAnclaje):
        if TipoDeAnclaje.esTipoDeAnclaje(tipoDeAnclaje):
            super().grid(sticky=tipoDeAnclaje.getValor())
        return self
    def setPading_Grid(self,*args):
        """
        (row,column)
        (rowYcolumn) un parametro row==column
        :param args:
        :return:
        """
        a=tuplaRectificada(args)
        leng =len(a)
        if leng==2 and esIntAll(a[0],a[1]):
            super().grid(padx=a[0],pady=a[1])
        elif leng==1 and esInt(a[0]):
            super().grid(padx=a[0], pady=a[0])
        return self


def new_Labelr_Grid(contenedor,row,column,texto,*a):
    """
    (contenedor, row, column)
    (contenedor, row, column,anclaje)
    (contenedor, row, column,anclaje,pading)
    (contenedor, row, column,anclaje,padingX,padingY)
    (contenedor, row, column,pading)
    (contenedor, row, column,padingX,padingY)
    :param contenedor:
    :param row:
    :param column:
    :param texto:
    :param a:
    :return:
    """
    return Labelr(contenedor,texto).setGrid(row,column,a)

def esLabelr(a):
    return isinstance(a, Labelr)



from Tkinters.Imports.VisualPythonBasicosImports import *
from tkinter import Tk
#from Utiles.Utiles import *
from Tkinters.ClasesUtiles.Componentes.FrameVentana import FrameVentana



class Ventana(Tk):
	#def __init__(self,ancho,alto):
	#	self._frameVentana = FrameVentana(self,ancho,alto)
	#	self._frameVentana.pack(fill="both")
	def __init__(self,*args):
		"""
		(ancho,alto)
		()vacio
		:param args:
		"""
		super().__init__()
		leng = len(args)
		#print(leng)
		#print("a0=",args[0],"a1=", args[1])
		#print(esIntAll(args[0], args[1]))
		if (leng == 2 and esIntAll(args[0], args[1])):
			#print("u n o")
			self._frameVentana = FrameVentana(self,args[0], args[1])
		else:
			#print("d o s")
			self._frameVentana=FrameVentana()

		self._frameVentana.pack(fill="both")


	def setBackgraund(self,nombreColor):
		super().config(bg=nombreColor)
		self._frameVentana.setBackgraund(nombreColor)
		return self
	def setTitle(self,titulo):
		super().title(titulo)
		return self
	def setSize(self,width,height):
		#super().geometry(str(str(width)+"x"+str(height)))
		self._frameVentana.setSize(width,height)
		return self
	def setIcon(self,direccion):
		if esString(direccion):
			super().iconbitmap(direccion)
		return self
	def setResizable(self,permitido):
		super().resizable(permitido,permitido)
		return self
	def setTipoDeBorde(self,borde):
		self._frameVentana.setTipoDeBorde(borde)
		return self
	def setBordeWidth(self,ancho):
		self._frameVentana.setBordeWidth(ancho)
		return self
	def setBorde(self,ancho,tipoDeBorde):
		self._frameVentana.setBorde(ancho,tipoDeBorde)
		return self
	def setCursor(self,tipoDeCursor):
		self._frameVentana.setCursor(tipoDeCursor)
		return self
	def getFrame(self):
		return self._frameVentana
	def getTk(self):
		return super()
	def show(self):
		super().mainloop()
		return self
	def setMenu(self,menu):
		if esMenu(menu):
			super().config(menu=menu)
	def showDialogoSalir(self):
		showDialogoSalir(self)

def esVentana(a):
	return isinstance(a,Ventana)


""""
pip3 uninstall ReneTkinter
pip3 install ReneTkinter-1.0.tar.gz
 python setup.py sdist
 
"""
#def hola(*a):
#	print(len(a))

#hola()
#a=4==2 and 4==3





from setuptools import setup
setup(

	name="ReneTkinter",
	install_requires=["RenePython"],
	version="1.1",
	description="visual de Python",
	author="René Lázaro Collado Arteaga",
	author_email="renearteaga261998@gmail.com",
	license="GPLv3",
	include_package_data=True,
	packages=["Tkinters"
		,"Tkinters.ClasesUtiles"
		,"Tkinters.ClasesUtiles.Tipos"
		,"Tkinters.ClasesUtiles.Componentes"
		,"Tkinters.ClasesUtiles.Interfaces"
		,"Tkinters.Imports"],
	url="https://github.com/critBus/ReneTkinter",
	classifiers = ["Programming Language :: Python :: 3",
	"License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
	"Development Status :: 4 - Beta", "Intended Audience :: Developers",
	"Operating System :: OS Independent"],
	
	)


from tkinter import Frame
from tkinter import Tk
from tkinter import StringVar
from tkinter import Menu
from tkinter import messagebox
from tkinter import filedialog
from Utiles.Utiles import *
#from Tkinters.ClasesUtiles.Ventana import esVentana

def esFrame(a):
    return isinstance(a,Frame)
def esTk(a):
    return isinstance(a,Tk)
def esStringVar(a):
    return isinstance(a,StringVar)
def append(a,text):
    if esStringVar(a):
        a.set(a.get()+text)
def esMenu(a):
    return isinstance(a,Menu)



def responderException(ex):
    mensaje="error de codigo interno"
    messagebox.showerror("Error", mensaje)
def showDialogoSalir(ventana):
    if esTk(ventana) and messagebox.askquestion("Salir","Desea cerrar la ventana")=="yes":
        ventana.destroy()
def showInfo(mensaje):
    messagebox.showinfo("Informacion",mensaje)
def showAdvertencia(mensaje):
    messagebox.showwarning("Advertencia",mensaje)
def showError(mensaje):
    messagebox.showerror("Error",mensaje)
def showAceptarCancelarInf(titulo,mensaje):
    return messagebox.askquestion(titulo,mensaje)=="yes"
def showAceptarCancelarAdvertencia(titulo,mensaje):
    return messagebox.askretrycancel(titulo,mensaje)
def showFileChoser(fileDefault="C:",*extenciones):
    a=tuplaRectificada(extenciones)

    if esString(fileDefault):
        leng2=len(fileDefault)
        if leng2==1:
            fileDefault=fileDefault.upper()+":"
        elif leng2<7 and (fileDefault.startswith(".") or fileDefault.startswith("*")):
            l=list(a)
            l.insert(0,fileDefault)
            a=tuple(l)
            fileDefault="C:"
    leng = len(a)
    fileType=None
    if leng>0:
        fileType=[]
        for i in a:
            if not i.startswith("*"):
                i=str("*")+i
            fileType.append((i,i))
        fileType=tuple(fileType)
    if fileType==None:
        fileType=(("todos","*.*"))
        #return filedialog.askopenfilename(title="titulo",initialdir=fileDefault)
    return filedialog.askopenfilename(title="Buscar archivos",initialdir=fileDefault,filetypes=fileType)
def showDirectoryChoser(fileDefault="C:"):
    return filedialog.askdirectory(title="Buscar carpetas",initialdir=fileDefault)



from Tkinters.Imports.VisualPythonBasicosImports import *
from Tkinters.ClasesUtiles.Componentes.Ventana import esVentana
from tkinter import Menu
class Menur(Menu):
    def __init__(self,contenedor,titulo=None):
        super().__init__(contenedor)
        self._parent = None
        if esVentana(contenedor):
            contenedor.setMenu(self)
        elif esTk(contenedor):
            contenedor.config(menu=self)
        elif esMenu(contenedor):
            self._parent=contenedor
        self._subMenus=[]
        self._titulo=titulo

    def addSubMenu(self,titulo):
        sub=Menur(self,titulo)
        sub.config(tearoff=0)
        self._subMenus.append(sub)
        super().add_cascade(label=titulo,menu=sub)
        return sub
    def addItemAccion(self,titulo,accion):
        super().add_command(label=titulo,command=accion)
        return self
    def getSubMenus(self):
        return self._subMenus
    def getTitulo(self):
        return self._titulo
    def getParent(self):
        return self._parent
def esMenur(a):
    return isinstance(a,Menur)






class TipoDeJustificacion:
    def __init__(self,valor):
        self.valor=valor
    def getValor(self):
        return self.valor

DERECHA=TipoDeJustificacion("right")
ISQUIERDA=TipoDeJustificacion("left")
CENTRO=TipoDeJustificacion("center")



values=(DERECHA,ISQUIERDA,CENTRO)

def esTipoDeJustificacion(a):
    return isinstance(a,TipoDeJustificacion)